#   DrCr: Web-based double-entry bookkeeping framework
#   Copyright (C) 2022–2023  Lee Yingtong Li (RunasSudo)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from flask import abort, redirect, render_template, request

from .. import AMOUNT_DPS
from ..database import db
from ..models import Posting, Transaction
from ..webapp import app
from .models import StatementLine, StatementLineReconciliation

@app.route('/statement-lines')
def statement_lines():
	if 'account' in request.args:
		statement_lines = db.select(StatementLine).where(StatementLine.source_account == request.args['account']).order_by(StatementLine.dt)
	else:
		statement_lines = db.select(StatementLine).order_by(StatementLine.dt)
	
	page = db.paginate(statement_lines, per_page=request.args.get('per_page', 100))
	
	return render_template('statements/statement_lines.html', page=page)

@app.route('/statement-lines/charge', methods=['POST'])
def statement_line_charge():
	statement_line = db.session.get(StatementLine, request.form['line-id'])
	if not statement_line:
		abort(404)
	
	if statement_line.reconciliation:
		raise Exception('NYI')
	
	# Create transaction
	posting_source = Posting(account=statement_line.source_account, quantity=statement_line.quantity, commodity=statement_line.commodity)
	posting_charge = Posting(account=request.form['charge-account'], quantity=-statement_line.quantity, commodity=statement_line.commodity)
	transaction = Transaction(
		dt=statement_line.dt,
		description=statement_line.description,
		postings=[posting_source, posting_charge]
	)
	db.session.add(transaction)
	
	# Reconcile statement line
	reconciliation = StatementLineReconciliation(
		statement_line=statement_line,
		posting=posting_source
	)
	db.session.add(reconciliation)
	
	db.session.commit()
	
	return 'OK'

@app.route('/statement-lines/reconcile-transfer', methods=['POST'])
def statement_line_reconcile_transfer():
	line_ids = request.form.getlist('sel-line-id')
	if len(line_ids) != 2:
		raise Exception('Must select exactly 2 statement lines')
	
	line1 = db.session.get(StatementLine, line_ids[0])
	line2 = db.session.get(StatementLine, line_ids[1])
	
	# Check same amount
	if line1.quantity != -line2.quantity or line1.commodity != line2.commodity:
		raise Exception('Selected statement line debits/credits must equal')
	
	# Delete existing postings
	line1.delete_postings()
	line2.delete_postings()
	
	transaction = StatementLineTransaction(
		dt=line1.dt,
		description=line1.description,
		postings=[
			StatementLinePosting(statement_line=line1, description=line1.description, account=line1.source_account, quantity=line1.quantity, commodity=line1.commodity),
			StatementLinePosting(statement_line=line2, description=line2.description, account=line2.source_account, quantity=line2.quantity, commodity=line2.commodity)
		]
	)
	db.session.add(transaction)
	db.session.commit()
	
	return redirect('/statement-lines')

@app.route('/statement-lines/import', methods=['GET', 'POST'])
def statement_lines_import():
	if request.method == 'GET':
		return render_template('statements/import.html')
	
	# Import using importer
	if request.form['format'] == 'ofx1':
		from .importers.ofx1 import import_ofx1
		statement_lines = import_ofx1(request.files['file'])
	else:
		abort(400)
	
	# Fill in source_account
	for statement_line in statement_lines:
		statement_line.source_account = request.form['source-account']
	
	if request.form['action'] == 'preview':
		return render_template('statements/import_preview.html', statement_lines=statement_lines)
	
	# Add to database
	for statement_line in statement_lines:
		db.session.add(statement_line)
	db.session.commit()
	
	return redirect('/statement-lines')
